/*
 * Copyright (C) 2004 Nathan Lutchansky <lutchann@litech.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

struct rtp_stream;
struct rtsp_session;

#define	RTP_TRANS_UDP	0
#define RTP_TRANS_INTER	1

struct rtp_endpoint {
	struct rtp_endpoint *next;
	struct rtp_endpoint *prev;
	struct rtp_stream *rtp_stream;
	struct rtsp_session *session;
	int payload;
	unsigned int ssrc;
	unsigned int timestamp;
	int seqnum;
	int packet_count;
	int octet_count;
	int running;
	int trans_type;
	union {
		struct {
			int rtp_fd;
			struct event *rtp_event;
			int rtcp_fd;
			struct event *rtcp_event;
		} udp;
		struct {
			void *conn;
			int chan;
		} inter;
	} trans;
};

struct rtp_stream {
	const char *(*get_sdp)( struct rtp_stream *rtp );
	void (*new_endpoint)( struct rtp_stream *rtp, struct rtp_endpoint *ep );
	void (*del_endpoint)( struct rtp_stream *rtp, struct rtp_endpoint *ep );
	struct rtp_endpoint *ep_list;
	void *private;
};

struct rtsp_session {
	struct rtsp_session *next;
	struct rtsp_session *prev;
	char id[32];
	struct rtp_endpoint *endpoint;
};

#define PUT_16(p,v) ((p)[0]=((v)>>8)&0xff,(p)[1]=(v)&0xff)
#define PUT_32(p,v) ((p)[0]=((v)>>24)&0xff,(p)[1]=((v)>>16)&0xff,(p)[2]=((v)>>8)&0xff,(p)[3]=(v)&0xff)
#define GET_16(p) (((p)[0]<<8)|(p)[1])
#define GET_32(p) (((p)[0]<<24)|((p)[1]<<16)|((p)[2]<<8)|(p)[3])

struct rtp_endpoint *new_rtp_udp_endpoint( struct rtp_stream *rtp,
		struct in_addr dest_ip, int dest_port, int *our_port );
struct rtp_endpoint *new_rtp_interleaved_endpoint( struct rtp_stream *rtp,
		void *conn, int chan );
void del_rtp_endpoint( struct rtp_endpoint *ep );
void update_rtp_timestamp( struct rtp_stream *rtp, int time_increment );
void send_rtp_packet( struct rtp_stream *rtp,
				unsigned char *d, int len, int marker );
